@@ -9,7 +9,7 @@ __email__ = "help@tappingstone.com"
 __copyright__ = "Copyright 2013, TappingStone, Inc."
 __license__ = "Apache License, Version 2.0"
 
-__version__ = "0.5.0"
+__version__ = "0.6.0"
 
 
 # import packages
@@ -63,6 +63,10 @@ class ItemRecNotFoundError(PredictionIOAPIError):
     "Error happened when tried to get item recommendation"
     pass
 
+class ItemSimNotFoundError(PredictionIOAPIError):
+    "Error happened when tried to get similar items"
+    pass
+
 class InvalidArgumentError(PredictionIOAPIError):
     "Arguments are not valid"
     pass
@@ -405,12 +409,12 @@ class Client:
                                       (response.request, response.status, response.body))
         return None
 
-    def _aget_user_itemrec_topn(self, uid, n, engine, params={}):
+    def _aget_user_itemrec_topn(self, engine, uid, n, params={}):
         """Private function to asynchronously get recommendations for user
 
+        :param engine: name of the prediction engine. type str.
         :param uid: user id. type str.
         :param n: number of recommendation. type int.
-        :param engine: name of the prediction engine. type str.
         :param params: optional parameters. type dictionary
                 For example, { 'pio_itypes' : ("t1","t2") }
         :returns:
@@ -452,11 +456,11 @@ class Client:
         data = json.loads(response.body) # convert json string to dict
         return data
 
-    def aget_itemrec_topn(self, n, engine, params={}):
+    def aget_itemrec_topn(self, engine, n, params={}):
         """Asynchronously get recommendations for the identified user
 
-        :param n: number of recommendation. type int.
         :param engine: name of the prediction engine. type str.
+        :param n: number of recommendation. type int.
         :param params: optional parameters. type dictionary
                 For example, { 'pio_itypes' : ("t1",) }
         :returns:
@@ -467,7 +471,7 @@ class Client:
         if self._uid is None:
             raise InvalidArgumentError("uid is not identified. Please call identify(uid) first.")
 
-        request = self._aget_user_itemrec_topn(self._uid, n, engine, params)
+        request = self._aget_user_itemrec_topn(engine, self._uid, n, params)
         return request
 
     def aget_itemrec(self, uid, n, engine, **params):
@@ -482,7 +486,70 @@ class Client:
             AsyncRequest object. You should call the aresp() method using this AsyncRequest
             object as argument to get the final result or status of this asynchronous request.
         """
-        request = self._aget_user_itemrec_topn(uid, n, engine, params)
+        request = self._aget_user_itemrec_topn(engine, uid, n, params)
+        return request
+
+    def _aget_itemsim_topn(self, engine, iid, n, params={}):
+        """Private function to asynchronously get top n similar items of the item
+
+        :param engine: name of the prediction engine. type str.
+        :param iid: item id. type str.
+        :param n: number of similar items. type int.
+        :param params: optional parameters. type dictionary
+                For example, { 'pio_itypes' : ("t1","t2") }
+        :returns:
+            AsyncRequest object. You should call the aresp() method using this AsyncRequest
+            object as argument to get the final result or status of this asynchronous request.
+        """
+        if "pio_itypes" in params:
+            params["pio_itypes"] = ",".join(params["pio_itypes"])
+        if "pio_latlng" in params:
+            params["pio_latlng"] = ",".join(map(str, params["pio_latlng"]))
+        if "pio_attributes" in params:
+            params["pio_attributes"] = ",".join(params["pio_attributes"])
+
+        enc_iid = urllib.quote(iid,"") # replace special char with %xx
+        path = "%s/engines/itemsim/%s/topn.json" % (self.apiversion, engine)
+        request = AsyncRequest("GET", path, pio_appkey=self.appkey, pio_iid=enc_iid, pio_n=n, **params)
+        request.set_rfunc(self._aget_itemsim_topn_resp)
+        self._connection.make_request(request)
+        return request
+
+    def _aget_itemsim_topn_resp(self, response):
+        """Private function to handle the AsyncResponse of the aget_itemsim request
+
+        :param response: AsyncResponse object
+
+        :returns:
+            data in dictionary format.
+
+        :raises:
+            ItemSimNotFoundError.
+        """
+        if response.error is not None:
+            raise ItemSimNotFoundError("Exception happened: %s for request %s" % \
+                                               (response.error, response.request))
+        elif response.status != httplib.OK:
+            raise ItemSimNotFoundError("request: %s status: %s body: %s" % \
+                                               (response.request, response.status, response.body))
+
+        data = json.loads(response.body) # convert json string to dict
+        return data
+
+    def aget_itemsim_topn(self, engine, iid, n, params={}):
+        """Asynchronously get top n similar items of the item
+
+        :param engine: name of the prediction engine. type str.
+        :param iid: item id. type str.
+        :param n: number of similar items. type int.
+        :param params: optional parameters. type dictionary
+                For example, { 'pio_itypes' : ("t1",) }
+        :returns:
+            AsyncRequest object. You should call the aresp() method using this AsyncRequest
+            object as argument to get the final result or status of this asynchronous request.
+        """
+
+        request = self._aget_itemsim_topn(engine, iid, n, params)
         return request
 
     def _auser_action_on_item(self, action, uid, iid, params):
@@ -758,11 +825,11 @@ class Client:
         result = self.aresp(request)
         return result
 
-    def get_itemrec_topn(self, n, engine, params={}):
+    def get_itemrec_topn(self, engine, n, params={}):
         """Blocking request to get recommendations for the identified user
 
-        :param n: number of recommendation. type int.
         :param engine: name of the prediction engine. type str.
+        :param n: number of recommendation. type int.
         :param params: optional parameters. type dictionary
                 For example, { 'pio_itypes' : ("t1", "t2") }
         :returns:
@@ -771,7 +838,7 @@ class Client:
         :raises:
             ItemRecNotFoundError.
         """
-        request = self.aget_itemrec_topn(n, engine, params)
+        request = self.aget_itemrec_topn(engine, n, params)
         result = self.aresp(request)
         return result
 
@@ -795,6 +862,25 @@ class Client:
         result = self.aresp(request)
         return result
 
+    def get_itemsim_topn(self, engine, iid, n, params={}):
+        """Blocking request to get top n similar items of the item
+
+        :param engine: name of the prediction engine. type str.
+        :param iid: item id. type str.
+        :param n: number of similar items. type int.
+        :param params: optional parameters. type dictionary
+                For example, { 'pio_itypes' : ("t1",) }
+        :returns:
+            data in dictionary format.
+
+        :raises:
+            ItemSimNotFoundError.
+        """
+
+        request = self.aget_itemsim_topn(engine, iid, n, params)
+        result = self.aresp(request)
+        return result
+
     def record_action_on_item(self, action, iid, params={}):
         """Blocking request to create action on an item
 
